;================================
;          ZX-Assembler-2 
;      by Artic Computing LTD
;================================

ORG 4084

;================================
;ROM and SYS vars
;================================
@Keyboard=02BB			;Keyboard scanning subroutine
@Decode=07BD			;Keyboard decode subroutine
@PrntAt=08F5			;The PRINT AT routine
@Laddr=09D8			;Line address subroutine
@Scanning=0F55			;Expression evaluator
@FPtoBC=0EA7			;16Bit FP to INT, results to BC.
@FPtoA=15CD			; 8Bit FP to INT, results to A.
@DFile=400C			;Address of display file
@LastK=4025			;Keyboard debounce value
@NxtLine=4029			;Next program line to be executed
@Seed=4032			;Random number seed
;================================
;PRBuff vars
;================================
@Objekt=403C			;16444, IY+60 Object found code
@Pauz=403D			;16445, IY+61 Inline pause value
@ZapIt=403E			;16446, IY+62 Object to be teleported
@Hero=4040			;16448, IY+64 Hero's Icon A,V,<,>
@OrbOn=4041			;16449, IY+65 Match Lit Flag
@ObjectXY=4042			;16450, IY+66 Object code at HeroXY
@ViewOrb=4043			;16451, IY+67 View orb flag
@Walls=4044			;16452, IY+68 Left cave chr
@HeroXY=4045			;16453, IY+69 Hero's current position
@NextXY=4047			;16455, IY+71 Hero's Next position
@Die1=4049			;16453, IY+73 Die 1 value
@Die2=404A			;16455, IY+74 Die 2 value
;================================
;Constants, 16K settings;  Set RAMTOP to 28672/7000
;================================
@Map=7000			;28672,Map location
@MapEnd=7317			;29463,Last map addr
@Row1=7023			;28707,Map Row1+1
@Row23=72F9			;29433,Map Row23
@CaveData=733A			;29498,Cave wall data
@Store=7E9E			;32414,Screen data
;================================
;Jump table
;================================
	JR MapTest		;16516, Test for valid map
	JP Teleport		;16518, Teleport Hero
	00 00 00
	JR Pause		;16524, Inline pause
	JP Objects		;16526, GOTO Object
	JR Release		;16529, Wait for key release
	JR Press		;16531, Wait for key press
	JR Rnd256		;16533, RND*256
	JP Zap			;16535, Teleport object
	JP Clr			;16538, Clear input area
	JP Waiting		;16541, Animated waiting for key press
	JP KeyLoop		;16544, Key move loop
	JP Turns		;16547, Turns sorter
	JP HeroMove		;16550, Hero movement
	JP Orb			;16553, Print Orbview
	JP CaveWall		;16556, Print Cave Wall
	JP Save			;16559, Save screen
	JP Load			;16562, Load screen
	JP Mapview		;16565, View map
	JP Dice			;16568, Dice roll
;================================
;Valid map test
;================================
@MapTest
	LD B,20			;Counter
	LD HL,Row23		;Start of Map row 23
	LD A,(HL)		;Get chr
	CP A6			;Hero's Icon?
	JR Z +5			;Jump if Hero found
	INC HL
	DJNZ MapTest+5
@ErrorM
	RST 08
	15			;Error Report "M", Map missing
	LD B,H			;Hero's addr
	LD C,L
	LD (HeroXY),HL		;Save hero's addr
	LD (IY+64),A6		;Save hero's icon
	RET			;Return Hero's addr
;================================
;Wait for release key 
;================================
@Release
	LD A,(LastK)		;Test for keypress
	CP FF			;Keypressed?
	JR NZ Release		;Yes, loop back
@Ret0
        LD BC,0000
	RET
;================================
;Press key 
;================================
@Press
	LD A,(LastK)		;Test for keypress
	CP FF			;Keypressed?
	JR Z Press		;No, loop back
	JR Ret0
;================================
; Pause replacement
;  Pressing a key aborts the pause
;================================
@Pause
	CALL Release		;Wait for key release
	CALL Delay		;Pause
	JR Release		;Return via key release
;================================
; Generic delay routine
;  POKE 16445,Delay value
;================================
@Delay
	LD B,(IY+61)		;Get delay value
	LD C,01			;Delay loop
@KeyTest
	LD A,(LastK)		;Test for key press
	CP FF
	RET NZ			;End delay if key press
	DEC BC 
	LD A,B 
	OR C 
	JR NZ KeyTest
	RET
;================================
; RND*256
;================================
@Rnd256
	CALL RNDNum
	LD B,00
	RET
;================================
;GOTO Object
;================================
@Objects
	LD HL,(HeroXY)
	LD A,(HL)
	LD HL,000A		;Line 10
	AND A			;0 test
	JR Z Nothing		;GOTO 100, nothing found
	LD HL,Codes		;Code/GOTO table addr
	LD BC,0024		;Table Byte count
	CPIR			;Compare code
	JR Z GotCode		;Jump if match
        BIT 6,(HL)		;EOT test
	JR ErrorM
@GotCode
	LD E,(HL)		;Get line number value
	INC HL
	LD D,(HL)
	EX DE,HL		;HL=Line number
@Nothing
	CALL LAddr		;Locate line number
	LD (NxtLine),HL		;GOTO line number
	RET
;=============================
; Teleport object routine
;=============================
@Zap
	CALL RND792		;Get random number, range 0 to 792
	LD HL,Map		;Map address
	ADD HL,BC		;Random Map addr
	LD A,(HL)		;0 test
	AND A			;Search for empty location
	JR NZ Zap		;Loop until empty location found
	LD A,(ZapIt)		;Get Zap Object
	LD (HL),A		;Fill empty location with object
	JP Ret0			;Return 0 to BASIC
;=============================
; Use Teleport Stone
;=============================
@Teleport
	CALL Zap		;Locate empty location
	LD (HL),A6		;Set hero icon
	LD B,H			;Set BC for return
	LD C,L
	RET
;================================
; Test for keypress.
; On exit...
;   A=chr code
;================================
@KeyScan
	CALL Keyboard		;Call keyboard scanning routine
	LD B,H 			;BC=Raw Key values
	LD C,L 
	LD D,C			;Test for keypress, D=255 if no key press
	INC D 			;0?
	RET Z
	CALL Decode		;Call keyboard decode routine, BC to ZX81 codes
	INC A			;Set Z flag
	LD A,(HL)
	RET			;Return chr code
;=============================
; Process key presses
;=============================
@KeyLoop
	CALL KeyScan		;Get key press
	JR Z KeyLoop		;No, loop until key pressed
	LD HL,Keys		;Code/GOTO table addr
	LD BC,0015		;Table Byte count
	CPIR			;Compare code
	JR Z KeyCode		;Jump if match
	JR KeyLoop		;Loop until valid keypress
@KeyCode
	LD E,(HL)		;Get line number value
	INC HL
	LD D,(HL)
	EX DE,HL		;HL=Line number
	CALL LAddr		;Locate line number
	LD (NxtLine),HL		;GOTO line number
	RET			;Ret 0 to BASIC
;================================
; Animated waiting for key press
;================================
@Waiting
	LD HL,(DFile)		;Get DFile
	LD DE,+780		;Offset to prompt message
	ADD HL,DE		;HL=Prompt location
	LD D,H			;DE=Prompt location
	LD E,L
@WaitLup
	PUSH HL			;PUSH Prompt location
	LD A,(HL)		;Get chr
	ADD A,80		;Add 128
	LD (HL),A		;Set new chr
	INC HL			;Next chr
	LD B,03			;Delay value
	CALL Delay+3		;Short Delay between chr prints
	BIT 6,(HL)		;N/L test
	JR Z WaitLup+1		;Inner Loop until N/L reached
	POP HL			;POP Prompt location
	LD BC,0000		;Set for return
	LD A,(LastK)		;Test for keypress
	CP 7F			;B-N-M-,-.?
	JR Z Clear		;End loop, return 00
	CP DF			;Y-U-I-O-P?
	JR NZ WaitLup		;Loop until Y or N
	LD C,01			;Return 01
@Clear
	EX DE,HL		;HL=Prompt location
	LD A,80			;Erase chr
	LD (HL),A		;Erase 
	INC HL			;Next addr
	BIT 6,(HL)		;N/L?
	JR Z Clear+1		;Loop until N/L reached
	RET			;Return 0 or 1 to BASIC
;================================
;Clear input area
;================================
@Clr
	LD HL,(DFile)		;Get DFile
	LD BC,015F		;Offset to NW corner of area to clear
	ADD HL,BC		;Source=DFile
	LD B,0E			;Row count
@ClrOuter
	LD C,0C			;Column count
@ClrInner
	LD (HL),00		;Clear chr
	INC HL			;Next DFile
	DEC C			;Decrease counter
	JR NZ ClrInner		;Loop until C=0
	LD DE,0015		;Offset to next row
	ADD HL,DE		;Next row addr
	DJNZ ClrOuter		;Loop until B=0
	LD C,B
	RET			;Return 0 to BASIC
;================================
;Load screen data from Store
;================================
@Load
	LD HL,771A		;H=LD (HL),A L=LD A,(DE)
	JR +3
;================================
;Save screen data from Store
;================================
@Save
	LD HL,127E		;H=LD (DE),A L=LD A,(HL)
	LD (SLInner),HL
;================================
	LD HL,(DFile)		;Get DFile
	LD BC,015F		;Offset to NW corner of area to clear
	ADD HL,BC		;Source=DFile
	LD DE,Store		;Destination=Store area
	LD B,0E			;Row count
@SLOuter
	LD C,0C			;Column count
@SLInner
	LD A,(HL)		;Get DFile chr
	LD (DE),A		;Save chr
	INC HL			;Next DFile
	INC DE			;Next Store
	DEC C			;Decrease counter
	JR NZ SLInner		;Loop until C=0
	LD A,B			;Save row counter
	LD BC,0015		;Offset to next row
	ADD HL,BC		;Next row addr
	LD B,A			;Restore row counter
	DJNZ SLOuter		;LoOp until B=0
	LD C,B
	RET
;================================
;View map
;================================
@MapView
	CALL Release		;Wait for key release
	CALL Swap		;Swap DFile and Map
	LD HL,(HeroXY)		;Get heros current map position
	LD DE,MAP		;Start of Map 
	AND A			;Clear CY
	SBC HL,DE		;HL=Offset value
	LD DE,(DFile)		;Get DFile
	ADD HL,DE		;Add offset
@FlashHero
	RES 7,(HL)		;Flash Hero's icon
	BIT 0,(IX+61)		;Delay, 20
	BIT 0,(IX+61)		;Delay, 20
	SET 7,(HL)
	LD A,(LastK)		;Test for key press
	CP FF
	JR Z FlashHero		;Loop until keypressed
	CALL Release		;Wait for key release
;================================
;Swap DFile and Map data
;================================
@Swap
	LD BC,0319		;LDI Byte count, 793
	LD HL,(DFile)		;LDI Source, DFile
	LD DE,Map		;LDI Destination, Map area
@SLoop
	LD A,(DE)		;Get Map chr
	LDI			;Copy DFile chr to Map area
	RET PO			;Return when BC=0
	DEC HL			;Backup DFile addr
	LD (HL),A		;Copy Map chr to DFile
	INC HL			;Restore DFile addr
	JR SLoop
;================================
; Turn Sorter
;  PRINT USR Turns;CODE "HERO",CODE "TURN"
;================================
@Turns
	RST 20			;Get next character
	CALL Scanning		;Evaluate [CODE "HERO"] expression
	CALL FPtoA		;Move results to A
	PUSH AF			;PUSH Hero Icon
	RST 20
	CALL Scanning		;Evaluate [CODE "TURN"] expression
	CALL FPtoA		;Move results to A
	CP "T"
	JR NZ +5		;Jump if not "T"
	LD HL,Turn180
	JR TurnLook		;"T" Search
	CP "L"
	JR NZ +5		;Jump if not "L"
	LD HL,TurnLeft
	JR TurnLook		;"L" Search
	CP "R"
	LD HL,TurnRite		;"R" Search
@TurnLook			;Search selected table for Hero's icon
	POP BC			;B=Hero icon
	LD A,B			;Search for Hero's icon
	LD BC,0004		;Counter
@TurnLup
	CPI
	JR Z +3
	INC HL			;Skip this one
	JR TurnLup		;Loop until Hero's icon found
	LD A,(HL)		;Get new Hero icon
	LD (Hero),A		;Set Hero Icon
	RST 08			;ON Error Continue Restart
	FF
;================================
; PRINT USE HEROMOVE,P,X
;  P=Current Positon, X=Hero Icon
;================================
@HeroMove
	XOR A
        LD (Walls),A		;Clear Wall byte
	RST 20			;Get next character
	CALL Scanning		;Evaluate [P] expression
	CALL FPtoBC		;Move results to BC
	LD (HeroXY),BC		;Set Hero XY addr
	LD A,(BC)		;Get Object XY code
	LD (ObjectXY),A		;Save Object XY code
	RST 20
	CALL Scanning		;Evaluate [X] expression
	CALL FPtoA		;Move results to A
	LD (Hero),A		;Set hero icon
	LD HL,(HeroXY)		;Get Current position
	LD D,A			;D=Hero icon
	LD E,80			;E=Wall chr
;C=Left, check L-F-R for objects depending on orientation A < > V
	CP A6			;"A"?
	LD BC,FFFF
	JR Z SetL		;P-1
	CP 92			;">" 
	LD C,DF
	JR Z SetL		;P-33
	CP 93			;"<"?
	LD BC,0021
	JR Z SetL		;P+33
	LD C,01			;"V", P+1
@SetL
	ADD HL,BC		;Left side
	LD A,(HL)		;Get cave chr
	CP E			;Wall chr
	JR NZ +4		;Jump if not wall
	SET 2,(IY+68)		;Set wall on the left bit
	LD A,D			;Restore hero icon
	LD HL,(HeroXY)		;Get Current position
;A=Front
	CP A6			;"A"?
	LD BC,FFDF
	JR Z SetF		;P-33
	CP 93			;"<", 
	LD C,FF
	JR Z SetF		;P-1
	CP 92			;">"?
	LD BC,0001
	JR Z SetF		;P+1
	LD C,21			;"V", P+33
@SetF
	ADD HL,BC		;Front side
	LD A,(HL)		;Get Next XY code
	LD (NextXY),A		;Save Next XY code
	CP E			;Wall chr
	JR NZ +4		;Jump if not wall
	SET 1,(IY+68)		;Set wall in front bit
	LD A,D			;Restore hero icon
	LD HL,(HeroXY)		;Get Current position
;B=Rite
	CP A6			;"A"?
	LD BC,0001
	JR Z SetR		;P+1
	CP 92			;">", 
	LD C,21
	JR Z SetR		;P+33
	CP 93			;"<"?
	LD BC,FFDF
	JR Z SetR		;P-33
	LD C,FF			;"V", P-1
@SetR
	ADD HL,BC		;Rite side
	LD A,(HL)		;Get cave chr
	CP E			;Wall chr
	JR NZ +4		;Jump if not wall
	SET 0,(IY+68)		;Set wall on the rite bit
;================================
;Print the cave wall
;================================
@CaveWall
	LD A,(Walls)		;Get wall number
	LD HL,(DFile)		;Get DFile
	LD BC,0022		;Offset to row 1
	ADD HL,BC
	EX DE,HL		;DE=DFile destination addr
	LD HL,CaveWalls		;Cave wall list of offsets
	LD C,1B			;Table size
	CPIR			;Locate wall no.
	LD C,(HL)		;Set BC to offset value
	INC HL
	LD B,(HL)
	LD HL,CaveData		;Get Base address
	ADD HL,BC		;HL=Wall source addr
	LD B,00
	LD A,12			;Row count=18
@WallLup
	LD C,12			;Column count=18
	LDIR
	EX DE,HL
	LD C,0F			;Offset to next row
	ADD HL,BC
	EX DE,HL
	DEC A
	JR NZ WallLup
;================================
;Setup for return to BASIC
;================================
	LD HL,044C		;GOTO 1100, Default line number
	LD A,(Walls)		;Get wall value
	CP 08
	JR NZ +5
	LD HL,1388		;GOTO 5000
	JR WallGo
	CP 03
	JR NZ +4
	LD L,7E			;Wall 03-GOTO 1150
	JR WallGo
	CP 06
	JR NZ +2
	LD L,92			;Wall 06-GOTO 1170
@WallGo
	CALL LAddr		;Locate line number
	LD (NxtLine),HL		;GOTO line number
	RST 08			;Execute (NxtLine)
	FF
;================================
; Print OrbView Display
;================================
@Orb
	BIT 0,(IY+67)		;Test orb bit
	JP Z Ret0		;Return 0 if orb unavailable
	LD HL,(DFile)		;Get DFile
	LD BC,01A3		;Offset to print location
	ADD HL,BC		;Print location
	EX DE,HL		;DE=DFile addr
	LD HL,(HeroXY)		;Get hero's map location
	LD BC,FFBB		;Offset to NW corner of Orb data
	ADD HL,BC		;HL=Cave addr
	LD B,05			;Row count
@Outer
	LD C,07			;Col count
@Inner
	LD (OrbC),HL		;Store cave addr
	LD (OrbD),DE		;Store DFile addr
	PUSH BC			;PUSH counters
	LD A,(HL)		;Get cave chr
	CP 76			;118?
	JR NZ NotNL
	LD A,(Hero)		;Get Hero's icon
	CP 92			;">"?
	JR NZ +3
	INC HL
	JR NotNL
	CP 93			;"<"?
	JR NZ +1
	DEC HL
@NotNL
	LD A,(HL)		;Get cave chr
	AND A			;Clear CY
	LD BC,Map		;Start of Map 
	SBC HL,BC		;HL<Map?
	JR C Pad		;Pad line if underflow, HL<Map
	LD BC,(OrbC)		;BC=cave addr
	AND A			;Clear CY
	LD HL,MapEnd		;Last map addr
	SBC HL,BC		;HL>MapEnd?
	JR C Pad		;Pad line if overflow, HL>=MapEnd
	LD HL,(OrbC)		;Get cave addr
	LD A,(HL)		;Get cave chr
	AND A			;0?
	JR Z PrintOrb		;If Z then Print " "
	CP 80			;Object test
	JR Z PrintOrb		;If Z then Print "::"
	CP 76			;Object test
	LD A,08
	JR Z PrintOrb		;If Z then Print "::"
	LD A,(HL)		;Get cave chr
	BIT 0,(IY+65)		;Orb reveal value
	JR NZ +2
	LD A,8F			;Print "?"
@PrintOrb
	LD (DE),A		;Print to DFile
	INC HL			;Next OrbC
	INC DE			;Next OrbD
	POP BC			;POP counters
	DEC C			;Decrease column count
	JR NZ Inner		;Loop until C=0
@NextRow
	LD A,B			;Save counter
	LD HL,(OrbC)		;Get cave addr
	LD BC,001B		;Offset to next row addr
	ADD HL,BC		;Start of next cave row
	EX DE,HL
	LD HL,(OrbD)		;Get DFile addr
	ADD HL,BC		;Start of next DFile row
	EX DE,HL
	LD B,A			;Restore counter
	DJNZ Outer		;Loop until B=0
	LD C,B
	LD HL,(DFile)		;Print Hero's icon
	LD DE,01E8		;Offset to hero's addr
	ADD HL,DE		;Hero's Orb addr
	LD A,(Hero)		;Get Hero's icon
	LD (HL),A		;Print hero's icon
	RET
;================================
; Stored cave & DFile Position addresses
;================================
@OrbC				;Orb cave addr
	00 00
@OrbD				;Orb DFile addr
	00 00
;================================
; If Under or Overflow pad Orb
; display with grey blocks
;================================
@Pad
	POP BC			;POP counters
	EX DE,HL		;HL=OrbD
	LD DE,(OrbC)
@Pad01
	LD A,(HL)		;Test for EOL chr
	CP 85
	JR NZ Pad02		;If not EOL print grey block
	DEC HL			;Adjust and store addresses
	DEC DE
	LD (OrbD),HL
	LD (OrbC),DE
	JR NextRow
@Pad02
	LD (HL),08		;Print grey
	INC HL			;Next DFile addr
	INC DE			;Next Cave addr
	JR Pad01		;Loop until EOL reached
;========================
@Dice
	LD HL,Die1		;Die1 addr
@Roll				;Roll the dice and store the results.
	LD B,02			;Number of dice to roll
@NextDie
	PUSH BC			;PUSH dice count
	PUSH HL			;PUSH Die1/2 address
	CALL RND256
	POP HL			;POP Die1/2 address
	POP BC			;POP dice count
@SixMax
	SUB 06
	JR NC SixMax
	CPL 			;Number=0 to 5
	INC A 			;Number=1 to 6
@DieNUm
	LD (HL),A		;Set Die1/2 value
	INC HL			;Die2 addr
	DJNZ NextDie
	LD A,(Die1)		;Die1 value
	LD HL,(DFile)
	LD BC,029A
	ADD HL,BC		;Die1 print position
	LD (PrintAt+1),HL	;Set print position
	PUSH HL			;PUSH Die1 print position
	CALL PrintDie
@DieTwo
	LD A,(Die2)		;Die2 value
	POP HL			;POP Die1 print position
	LD BC,0005
	ADD HL,BC		;Die2 print position
	LD (PrintAt+1),HL	;Set Die2 print position
	CALL PrintDie
	LD A,(Die1)
	ADD A,(IY+74)
	LD C,A
	RET			;Return total to BASIC
;================================
@PrintDie
	LD HL,Faces
	LD BC,003C
	CPIR
	LD A,03			;Counter, 3 row to print
@PrintAt
	LD DE,0000		;DE=Destination, print position
	PUSH DE			;PUSH print position
	LD C,03			;Byte count
	LDIR			;Print row
	EX DE,HL		;Swap for ADD
	POP HL			;POP print posion
	LD C,21			;Offset to next row
	ADD HL,BC		;Next row print position
	EX DE,HL		;Restore source addr
	DEC A			;Decrease counter
	JR NZ PrintAt+3
	RET
;=============================
;Generate a random number between 0 and 32768
;This routine extracted from MCoder II
;
; BC=RND*32768
;=============================
@RNDNum
	LD DE,(Seed)		;Get (Seed)
	LD H,E 			;H=Seed LO
	LD L,FD 		;L=253
	LD A,D 			;A=Seed HI
	OR A 			;Reset CY flag
	LD B,00 		;Clear B
	SBC HL,DE 		;Subtract seed from HL
	SBC A,B 		;Subtract CY from A
	SBC HL,DE  		;Subtract seed from HL again
	SBC A,B  		;Subtract CY from A again
	LD E,A 			;E=A
	LD D,B 			;Clear D
	SBC HL,DE		;Subtract DE from HL 
	JR NC +1		;If HL positive, set random number now
	INC HL 			;Otherwise increase HL first
	LD (Seed),HL		;Set (Seed) value
	LD B,H			;BC=0 to 32767
	LD C,L
	LD A,L			;A=0 to 255
	RET
;=============================
;This section limits the range to 0 to 792
;=============================
@RND792
	CALL RNDNum		;Get random number
	LD B,00			;Limit range to 0->792
	LD C,L			;C=LSB
	LD A,H			;A=MSB
	CP C0			;Test MSB
	JR C MSB2		;Jump if MSB<768
	LD B,03			;B=768
	LD A,C			;Test LSB
	CP 18
	JR C +2			;Jump if LSB<24
	LD A,18			;A=24
	LD C,A			;Set LSB
	RET			;BC=768 to 792
@MSB2
	CP 80			;Test MSB
	JR C MSB1		;Jump if MSB<512
	LD B,02			;B=512
	RET			;BC=512 to 767
@MSB1
	CP 40			;Test MSB
	JR C +2			;Jump if MSB<256
	LD B,01			;B=256
	RET			;BC=0 to 511
;================================
; Lookup Tables
;================================
@Faces
;Die   Face
;---   ----
  01 80 80 80
     80 97 80
     80 80 80
  02 80 80 97
     80 80 80
     97 80 80
  03 80 80 97
     80 97 80 
     97 80 80
  04 97 80 97
     80 80 80
     97 80 97
  05 97 80 97
     80 97 80
     97 80 97
  06 97 80 97
     97 80 97
     97 80 97 
;================================
@Keys
;================================
;IF INKEY$="?" THEN GOTO XXXX
;================================
;CODE  GOTO
;----  ----
  36   01F4	 		;500,  Forward	    Q-64
  26   0258			;600,  Turn Around  A-38
  34   02BC			;700,  Turn Left    O-52
  35   02C6 			;710,  Turn Rite    P-53
  3B   02D0 			;720,  View Map     V-59
  37   02DA			;730,  Orb Reveal   R-55
  39   02EE			;750,  Teleport	    T-57
  FF				;EOT
;================================
@Codes
;================================
;IF Object=? THEN GOTO XXXX
;================================
;CODE  GOTO
;----  ----
  B2   07D0 			;2000, Minotaur	   gM-178
  B8   0C80 			;3200, Spell	   gS-184
  B9   0BB8			;3000, Treasure	   gT-185
  B5   0C1C 			;3100, Potion	   gP-181
  97   0CE4 			;3300, *Gold	   g*-151
  B0   0D48 			;3400, Kaslo Key   gK-176
  AC   0F3C			;3900, Gremlin	   gG-172
  B1   0DAC 			;3500, Locked Door gL-183
  B7   0E10 			;3600, Rattler	   gR-183
  AB   0E74			;3700, Ferryman    gF-171
  8C   0ED8			;3800, Frog	   g-171
  8A   0FA0			;4000, Maze Exit   g:-138, Grey/Black
  FF				;EOT
;================================
; Turns
;================================
@Turn180			;Turn 180 degrees
  A6 BB				;IF A THEN V
  BB A6				;IF V THEN A
  92 93				;IF > THEN <
  93 92				;IF < THEN >
@TurnLeft			;Turn Left
  A6 93 			;IF A THEN <
  93 BB				;IF < THEN V
  BB 92				;IF V THEN >
  92 A6				;IF > THEN A
@TurnRite			;Turn Rite
  A6 92 			;IF A THEN >
  92 BB 			;IF > THEN V
  BB 93 			;IF V THEN <
  93 A6 			;IF < THEN A
;================================
@CaveWalls
;Wall  Offset
;----  ------
  08   0A20			;Splash 2592
  07   08DC			;W-W-W	2268
  03   0798			;L-W-W	1944
  05   03CC			;W-F-W	0972
  02   0510			;L-W-R	1296
  00   0288			;L-F-R	0648	
  01   0000			;L-F-W	0000
  04   0144			;W-F-R	0324
  06   0654			;W-W-R	1620
  FF				;EOT marker