;================================
;          ZX-Assembler-2 
;      by Artic Computing LTD
;================================
;ROM routines
@Scanning=0F55			;Expression evaluator
@FPtoA=15CD			; 8Bit FP to INT, results to A.
@DFile=400C			;Address of display file
@LastK=4025			;Keyboard debounce value
;Constants
@Data=7000
@Maze=+16515			;Cave Maze
@Walls=+17341			;Cave walls

ORG 4F32

;================================
;Jump table
;================================
	JR Save			;20274, Save data to 1 REM
	JR Load			;20276, Load data from 1 REM
	JR MapView		;20278, View Map
	JR WallView		;20280, Wall Viewer
;================================
;Save cave data to REM 1
;================================
@Save
	LD HL,Data		;Set source addr
	LD DE,Maze		;Set destination addr
	LD BC,0E9F		;Set byte count
	LDIR			;Copy data to 1 REM or 8192
	RET			;Ret 0 to BASIC
;================================
;Load cave data to 28672/7000
;================================
@Load
	LD HL,Maze		;Set source addr
	LD DE,Data		;Set destination addr
	JR Save+6
;================================
;View/Edit map
;================================
@MapView
	CALL Release
	LD HL,(DFile)		;Get DFile addr
	LD BC,Maze		;Maze addr
	LD (DFile),BC		;Set DFile to map addr
	CALL Press		;Wait for key press
	LD (DFile),HL		;Restore DFile
@Ret0
	LD BC,0000		
	RET			;Ret 0 to BASIC
;================================
;Wait for release key 
;================================
@Release
	LD A,(LastK)		;Test for keypress
	CP FF			;Keypressed?
	JR NZ Release		;Yes, loop back
	JR Ret0
;================================
;Press key 
;================================
@Press
	LD A,(LastK)		;Test for keypress
	CP FF			;Keypressed?
	JR Z Press		;No, loop back
	JR Ret0
;================================
; PRINT USR PrintWall,WallNo
;================================
@WallView
	RST 20			;Get next character
	CALL Scanning		;Evaluate [WallNo] expression
	CALL FPtoA		;Move results to A
	LD HL,(DFile)		;Get DFile
	LD BC,0022		;Offset to row 1
	ADD HL,BC
	EX DE,HL		;DE=DFile destination addr
	LD HL,WallList		;Wall list table of offsets
	LD C,1B			;Table size
	CPIR			;Locate wall no.
	LD C,(HL)		;Set BC to offset value
	INC HL
	LD B,(HL)
	LD HL,Walls		;Get Base address
	ADD HL,BC		;HL=Wall source addr
	LD B,00
	LD A,12			;Row count=18
@WallLup
	LD C,12			;Column count=18
	LDIR
	EX DE,HL
	LD C,0F			;Offset to next row
	ADD HL,BC
	EX DE,HL
	DEC A
	JR NZ WallLup
	RST 08
	FF
@WallList
;	No Offset		;W=Wall	Offset
	08 0A20			;Splash 2592
	07 08DC			;W-W-W	2268
	03 0798			;L-W-W	1944
	05 03CC			;W-F-W	0972
	02 0510			;L-W-R	1296
	00 0288			;L-F-R	0648	
	01 0000			;L-F-W	0000
	04 0144			;W-F-R	0324
	06 0654			;W-W-R	1620
	FF			;EOT marker

